const requestSchemas = {
  RegisterRequestSchema: {
    request: {
      cache: {
        validateInputs: "",
        insertOneResource: "",
        createJwtToken: "",
        addCookies: "",
      },
      action: [
        {
          validateInputs: {
            schema: "req.body.entitySchema",
            data: "req.body.data",
          },
        },
        {
          insertOneResource: {
            schema: "req.body.entitySchema",
            data: "cache.validateInputs",
          },
        },
        {
          createJwtToken: { id: "cache.insertOneResource.insertedId" },
        },
        { addCookies: { id: "cache.createJwtToken", res: "res" } },
      ],
    },
    response: {
      action: [
        {
          stringify: {
            data: "cache.addCookies",
          },
        },
      ],
    },
  },
  LoginRequestSchema: {
    request: {
      cache: {
        validateInputs: "",
        getOneResource: "",
        createJwtToken: "",
        addCookies: "",
      },
      action: [
        {
          validateInputs: {
            schema: "req.body.entitySchema",
            data: "req.body.data",
          },
        },
        {
          getOneResource: {
            schema: "req.body.entitySchema",
            data: "cache.validateInputs",
          },
        },
        {
          createJwtToken: { id: "cache.getOneResource._id" },
        },
        { addCookies: { id: "cache.createJwtToken", res: "res" } },
      ],
    },
    response: {
      action: [
        {
          stringify: {
            data: "cache.addCookies",
          },
        },
      ],
    },
  },
  CreateRequestSchema: {
    request: {
      cache: {
        validateInputs: "",
        addOwner: "",
        insertOneResource: "",
      },
      action: [
        {
          validateInputs: {
            schema: "req.body.entitySchema", // Task
            data: "req.body.data", //
          },
        },
        {
          decodeBearerToken: {
            Headers: "req.headers",
          },
        },
        {
          addOwner: {
            data: "cache.validateInputs",
            owner: "cache.decodeBearerToken._id",
          },
        },
        {
          insertOneResource: {
            schema: "req.body.entitySchema",
            data: "cache.validateInputs",
          },
        },
      ],
    },
    response: {
      action: [
        {
          stringify: {
            data: "cache.addCookies",
          },
        },
      ],
    },
  },
  ShowMeRequestSchema: {
    request: {
      cache: {},
      action: [
        {
          decodeBearerToken: {
            Headers: "req.headers",
          },
        },
        {
          getOneResourceById: {
            schema: "req.body.entitySchema",
            data: "cache.decodeBearerToken._id",
          },
        },
        {
          filterData: {
            action: "req.body.action",
            schema: "req.body.entitySchema",
            data: "cache.getOneResourceById",
          },
        },
      ],
    },
    response: {
      action: [
        {
          stringify: {
            data: "cache.addCookies",
          },
        },
      ],
    },
  },
  EditRequestSchema: {
    request: {
      cache: {
        validateInputs: "",
        addOwner: "",
        insertOneResource: "",
      },
      action: [
        {
          validateInputs: {
            schema: "req.body.entitySchema", // Task
            data: "req.body.data", //
          },
        },
        {
          decodeBearerToken: {
            Headers: "req.headers",
          },
        },
        {
          getOneResourceById: {
            schema: "req.body.entitySchema",
           id: "req.body.id",
          },
        },
        {
          checkRolesOverEntity: {
            schema: "req.body.entitySchema",
            data:  "cache.getOneResourceById",
            user: "cache.decodeBearerToken._id",
          },
        },
        {
          updateOneResourceById:{
            schema: "req.body.entitySchema",
            data: "cache.validateInputs"
          }
        }

      ],
    },
    response: {
      action: [
        {
          stringify: {
            data: "cache.addCookies",
          },
        },
      ],
    },
  }
};

export { requestSchemas };
