const htmlDomTojsonSchema = {
	tagName: {
	  type: "String",
	},
	nodeType: {
	  type: "Number",
	},
	textContent: {
	  type: "String",
	},
	attributes: {
	  type: "Object",
	  keys: {
	    id: {
	      type: "String",
	    },
	    scope: {
	      type: "String",
	    },
	    class: {
	      type: "String",
	    },
	    usemap: {
	      type: "String",
	    },
	    name: {
	      type: "String",
	    },
	    shape: {
	      type: "String",
	    },
	    value: {
	      type: "String",
	    },
	    src: {
	      type: "String",
	    },
	    srcset: {
	      type: "String",
	    },
	    list: {
	      type: "String",
	    },
	    media: {
	      type: "String",
	    },
	    width: {
	      type: "String",
	    },
	    height: {
	      type: "String",
	    },
	    coords: {
	      type: "String",
	    },
	    kind: {
	      type: "String",
	    },
	    srclang: {
	      type: "String",
	    },
	    target: {
	      type: "String",
	    },
	    style: {
	      type: "String",
	    },
	    for: {
	      type: "String",
	    },
	    type: {
	      type: "String",
	    },
	    data: {
	      type: "String",
	    },
	    placeholder: {
	      type: "String",
	    },
	    checked: {
	      type: "Boolean",
	    },
	    open: {
	      type: "Boolean",
	    },
	    controls: {
	      type: "Boolean",
	    },
	    rows: {
	      type: "Number",
	    },
	    cite: {
	      type: "String",
	    },
	    href: {
	      type: "String",
	    },
	    alt: {
	      type: "String",
	    },
	    max: {
	      type: "String",
	    },
	    rows: {
	      type: "String",
	    },
	    cols: {
	      type: "String",
	    },
	    viewBox: {
	      type: "String",
	    },
	    xmins: {
	      type: "String",
	    },
	    stroke: {
	      type: "String",
	    },
	    fill: {
	      type: "String",
	    },
	    cx: {
	      type: "String",
	    },
	    cy: {
	      type: "String",
	    },
	    r: {
	      type: "String",
	    },
	    req: {
	      type: "String",
	    },
	  },
	},
	children: {
	  type: "Array",
	},
      };
      // Rest of the api's to be created. As per doc/ehh-core/ehhApi.txt
      
      export class actionEntity {
	constructor() {}
	create() {}
	// Please add in Comments, small description of the method, and it's arguments. we need to have a schema of args of each methods. So that all the arguments can be validated using it.
	static insert(parent, key, data, schema) {
	  const dataType = resolveDataType(data);
	  switch (dataType) {
	    case "string":
	      parent[key] = data;
	      break;
	    case "number":
	      parent[key] = data;
	      break;
	    case "object":
	      const keys = schema[key]["keys"];
	      parent[key] = {};
	      for (const item in keys) {
		if (data[item]) {
		  let res;
		  parent[key][item] = {};
		  res = data[item]["value"];
		  if (res[0] === "{") {
		    res = JSON.parse(res);
		    parent[key][item]["value"] = res;
		    continue;
		  }
		  if (item["type"] == "Boolean") {
		    parent[key][item]["value"] = "1";
		    continue;
		  }
		  actionEntity.insert(parent[key][item], "value", res);
		}
	      }
	      break;
	    case "array":
	      parent[key] = [];
	      for (let index = 0; index < data.length; index++) {
		const child = data[index];
		let res = actionEntity.copyAs(schema, child);
		parent[key].push(res);
	      }
	      break;
	  }
	}
	// Please add in Comments, small description of the method, and it's arguments. we need to have a schema of args of each methods. So that all the arguments can be validated using it.
	static copyAs(schema, data) {
	  try {
	    let response = {};
	    for (const key in schema) {
	      const dataType = schema[key]["type"];
	      let validate;
	      switch (dataType) {
		case "String":
		  validate = schema[key]["validator"] || [];
		  for (let j = 0; j < validate.length; j++) {
		    let validation = validate[j];
		    let res = Validator[validation](data[key]);
		    if (!res)
		      throw new Error(
			"validation of data  " +
			  datas[index] +
			  " for key  " +
			  key +
			  "failed"
		      );
		  }
		  actionEntity.insert(response, key, data[key]);
		  break;
		case "Number":
		  validate = schema[key]["validator"] || [];
		  for (let j = 0; j < validate.length; j++) {
		    let validation = validate[j];
		    let res = Validator[validation](data[key]);
		    if (!res)
		      throw new Error(
			"validation of data  " +
			  datas[index] +
			  " for key  " +
			  key +
			  "failed"
		      );
		  }
		  actionEntity.insert(response, key, data[key]);
		  break;
		// Client side only
		case "Object":
		  actionEntity.insert(response, key, data[key], schema);
		  break;
		// Not Client Side only
		case "Array":
		  actionEntity.insert(response, key, data[key], schema);
		  break;
		// server side only
		case "[Model]":
		  let newSchema = schema[key].model;
		  let newData = data[key];
		  response[key] = [];
      
		  for (let index = 0; index < newData.length; index++) {
		    const child = newData[index];
		    let res = actionEntity.copyAs(newSchema, child);
		    response[key].push(res);
		  }
		  break;
		// server side only
		case "Model":
		  let newModelSchema = schema[key].model;
		  let newModelData = data[key];
		  response[key] = actionEntity.copyAs(newModelSchema, newModelData);
		  break;
		//server side only
		case "[]":
		  response[key] = [];
		  let datas = data[key];
		  validate = schema[key]["validator"];
		  for (let index = 0; index < datas.length; index++) {
		    const data = datas[index];
		    for (let j = 0; j < validate.length; j++) {
		      let validation = validate[j];
		      let res = Validator[validation](datas[index]);
		      if (!res)
			throw new Error(
			  "validation of data  " +
			    datas[index] +
			    " for key  " +
			    key +
			    "failed"
			);
		      response[key].push(data);
		    }
		  }
		  break;
		default:
		  break;
	      }
	    }
	    return response;
	  } catch (error) {
	    console.log(error);
	  }
	  /*  schema = "name of schema / model" , data = corresponding data to be converted   */
	}
	// Please add in Comments, small description of the method, and it's arguments. we need to have a schema of args of each methods. So that all the arguments can be validated using it.
	static json2html(json) {
	  var response = "";
	  if (json.nodeType == 1) {
	    response += `<${json.tagName}`;
	    for (const key in json.attributes) {
	      response += " ";
	      response += key;
	      if (typeof json.attributes[key].value == "string") {
		response += `= "${json.attributes[key].value}"`;
	      } else if (typeof json.attributes[key].value == "object") {
		response += `= '${JSON.stringify(json.attributes[key].value)}'`;
	      }
	    }
	    response += ">";
	    if ((!json.children || !json.children[0]) && json.textContent) {
	      json.textContent = specialToUnicode(json.textContent);
	      response += json.textContent;
	    }
	    if (json.children) {
	      let children = json.children;
	      for (let index = 0; index < children.length; index++) {
		const child = children[index];
		let res = actionEntity.json2html(child);
		response += res;
	      }
	    }
	    if (isSelfClosingTag(json.tagName)) {
	      response = response.slice(0, -1);
	      response += `/>\n`;
	    } else {
	      response += `</${json.tagName}>\n`;
	    }
	  }
	  return response;
	}
      }